@@ -1,5 +1,6 @@
 ;;; gtags.el --- gtags facility for Emacs
 
+;(setq debug-on-error t)
 ;;
 ;; Copyright (c) 1997, 1998, 1999, 2000, 2006, 2007, 2008, 2009, 2010
 ;;		2011
@@ -23,9 +24,9 @@
 
 ;; GLOBAL home page is at: http://www.gnu.org/software/global/
 ;; Author: Tama Communications Corporation
-;; Version: 3.2
+;; Version: 3.3
 ;; Keywords: tools
-;; Required version: GLOBAL 5.9.7 or later
+;; Required version: GLOBAL 5.9.7 or later.
 
 ;; Gtags-mode is implemented as a minor mode so that it can work with any
 ;; other major modes. Gtags-select mode is implemented as a major mode.
@@ -167,6 +168,7 @@
       (define-key gtags-mode-map "\C-cs" 'gtags-find-symbol)
       (define-key gtags-mode-map "\C-cr" 'gtags-find-rtag)
       (define-key gtags-mode-map "\C-ct" 'gtags-find-tag)
+      (define-key gtags-mode-map "\C-cd" 'gtags-find-tag)
       (define-key gtags-mode-map "\C-cv" 'gtags-visit-rootdir)
       ; common
       (define-key gtags-mode-map "\e*" 'gtags-pop-stack)
@@ -222,7 +224,84 @@
         (define-key gtags-select-mode-map [mouse-3] 'gtags-pop-stack)
         (define-key gtags-select-mode-map [mouse-2] 'gtags-select-tag-by-event)))
 )
-;; This is only one exception of the policy.
+
+;;
+;; TRAMP support
+;;
+;
+; TRAMP style default-directory.
+; /<method>:[<user id>@]<host name>:<directory>
+; Ex: /ssh:remoteuser@remotehost:/usr/src/sys/
+;
+(defconst gtags-tramp-path-regexp "^/\\([^:]+\\):\\([^:]+\\):\\(.*\\)"
+  "Regexp matching tramp path name.")
+(defconst gtags-tramp-user-host-regexp "^\\([^@]+\\)@\\(.*\\)"
+  "Regexp matching tramp user@host name.")
+(defvar gtags-tramp-active nil
+  "TRAMP activity.")
+(defvar gtags-tramp-saved-global-command nil
+  "Save area of the command name of global.")
+
+; The substitute of buffer-file-name
+(defun gtags-buffer-file-name ()
+  (if (string-match gtags-tramp-path-regexp buffer-file-name)
+      (match-string 3 buffer-file-name)
+      buffer-file-name))
+
+(defun gtags-push-tramp-environment ()
+    (let ((tramp-path default-directory))
+      (if (string-match gtags-tramp-path-regexp tramp-path)
+          (let ((shell         (match-string 1 tramp-path))
+                (user-and-host (match-string 2 tramp-path))
+                (cwd           (match-string 3 tramp-path)))
+            ;
+            ; Server side GLOBAL cannot treat other than rsh and ssh.
+            ;
+            (cond
+             ((equal shell "rsh"))
+             ((equal shell "ssh"))
+             ((equal shell "rcp")
+              (setq shell "rsh"))
+             ((equal shell "scp")
+              (setq shell "ssh"))
+             (t
+               (setq shell "ssh")))
+            (let (host user)
+              (if (string-match gtags-tramp-user-host-regexp user-and-host)
+                  (progn
+                    (setq user (match-string 1 user-and-host))
+                    (setq host (match-string 2 user-and-host)))
+                  (progn
+                    (setq user nil)
+                    (setq host user-and-host)))
+              ;
+              ; Move to tramp mode only when all the items are assembled.
+              ;
+              (if (and shell host cwd)
+                  (progn
+                    (setq gtags-tramp-active t)
+                    (setq gtags-tramp-saved-global-command gtags-global-command)
+                    ; Use 'global-client even if environment variable GTAGSGLOBAL is set.
+                    ;(setq gtags-global-command (getenv "GTAGSGLOBAL"))
+                    ;(if (or (not gtags-global-command) (equal gtags-global-command ""))
+                        (setq gtags-global-command "global-client")
+                    ;)
+                    (push (concat "GTAGSREMOTESHELL=" shell) process-environment)
+                    (push (concat "GTAGSREMOTEHOST="   host) process-environment)
+                    (push (concat "GTAGSREMOTEUSER="   user) process-environment)
+                    (push (concat "GTAGSREMOTECWD="     cwd) process-environment))))))))
+
+(defun gtags-pop-tramp-environment ()
+  (if gtags-tramp-active
+      (progn
+        (setq gtags-tramp-active nil)
+        (setq gtags-global-command gtags-tramp-saved-global-command)
+        (pop process-environment)
+        (pop process-environment)
+        (pop process-environment)
+        (pop process-environment))))
+
+;; End of TRAMP support
 
 ;;
 ;; utility
@@ -277,6 +356,8 @@
 ;; completsion function for completing-read.
 (defun gtags-completing-gtags (string predicate code)
   (gtags-completing 'gtags string predicate code))
+(defun gtags-completing-grtags (string predicate code)
+  (gtags-completing 'grtags string predicate code))
 (defun gtags-completing-gsyms (string predicate code)
   (gtags-completing 'gsyms string predicate code))
 (defun gtags-completing-files (string predicate code)
@@ -289,6 +370,7 @@
   ; The purpose of using the -n option for the -P command is to exclude
   ; dependence on the execution directory.
   (let ((option (cond ((eq flag 'files)   "-cPo")
+                      ((eq flag 'grtags)  "-cr")
                       ((eq flag 'gsyms)   "-cs")
                       ((eq flag 'idutils) "-cI")
                       (t                  "-c")))
@@ -298,7 +380,9 @@
         (setq option (concat option "i")))
     ; build completion list
     (set-buffer (generate-new-buffer "*Completions*"))
+    (gtags-push-tramp-environment)
     (call-process gtags-global-command nil t nil option string)
+    (gtags-pop-tramp-environment)
     (goto-char (point-min))
     ;
     ; The specification of the completion for files is different from that for symbols.
@@ -350,11 +434,11 @@
   "Tell tags commands the root directory of source tree."
   (interactive)
   (let (path input n)
-    (if gtags-rootdir
-      (setq path gtags-rootdir)
-     (setq path (gtags-get-rootpath))
-     (if (equal path nil)
-       (setq path default-directory)))
+    (setq path gtags-rootdir)
+    (if (not path)
+        (setq path (gtags-get-rootpath)))
+    (if (not path)
+        (setq insert-default-directory (if (string-match gtags-tramp-path-regexp default-directory) nil t)))
     (setq input (read-file-name "Visit root directory: " path path t))
     (if (equal "" input) nil
       (if (not (file-directory-p input))
@@ -390,7 +474,7 @@
    (if tagname
      (setq prompt (concat "Find tag (reference): (default " tagname ") "))
     (setq prompt "Find tag (reference): "))
-   (setq input (completing-read prompt 'gtags-completing-gtags
+   (setq input (completing-read prompt 'gtags-completing-grtags
                  nil nil nil gtags-history-list))
    (if (not (equal "" input))
      (setq tagname input))
@@ -459,7 +543,7 @@
   (interactive)
   (let (tagname prompt input)
     (setq prompt "Parse file: ")
-    (setq input (read-file-name prompt buffer-file-name buffer-file-name t))
+    (setq input (read-file-name prompt (gtags-buffer-file-name) (gtags-buffer-file-name) t))
     (if (or (equal "" input) (not (file-regular-p input)))
         (message "Please specify an existing source file.")
        (setq tagname input)
@@ -485,7 +569,7 @@
       (message "This is a null file.")
       (if (not buffer-file-name)
           (message "This buffer doesn't have the file name.")
-          (call-process "gozilla"  nil nil nil (concat "+" (number-to-string (gtags-current-lineno))) buffer-file-name))))
+          (call-process "gozilla"  nil nil nil (concat "+" (number-to-string (gtags-current-lineno))) (gtags-buffer-file-name)))))
 
 ; Private event-point
 ; (If there is no event-point then we use this version.
@@ -570,7 +654,7 @@
     (if case-fold-search
         (setq option (concat option "i")))
     (if (char-equal flag-char ?C)
-        (setq context (concat "--from-here=" (number-to-string (gtags-current-lineno)) ":" buffer-file-name))
+        (setq context (concat "--from-here=" (number-to-string (gtags-current-lineno)) ":" (gtags-buffer-file-name)))
         (setq option (concat option flag)))
     (cond
      ((char-equal flag-char ?C)
@@ -578,8 +662,7 @@
      ((char-equal flag-char ?P)
       (setq prefix "(P)"))
      ((char-equal flag-char ?f)
-      (setq prefix "(F)")
-      (setq option (concat option "q")))
+      (setq prefix "(F)"))
      ((char-equal flag-char ?g)
       (setq prefix "(GREP)"))
      ((char-equal flag-char ?I)
@@ -617,53 +700,58 @@
               (setq now-buffer-list (cdr now-buffer-list))))))
     (setq buffer (generate-new-buffer (generate-new-buffer-name (concat "*GTAGS SELECT* " prefix tagname))))
     (set-buffer buffer)
-    ;
-    ; Path style is defined in gtags-path-style:
-    ;   root: relative from the root of the project (Default)
-    ;   relative: relative from the current directory
-    ;	absolute: absolute (relative from the system root directory)
-    ;
-    (cond
-     ((equal gtags-path-style 'absolute)
-      (setq option (concat option "a")))
-     ((equal gtags-path-style 'root)
-      (let (rootdir)
-        (if gtags-rootdir
-          (setq rootdir gtags-rootdir)
-         (setq rootdir (gtags-get-rootpath)))
-        (if rootdir (cd rootdir)))))
     (message "Searching %s ..." tagname)
-    (if (not (= 0 (if (equal flag "C")
-                      (call-process gtags-global-command nil t nil option "--encode-path=\" \t\"" context tagname)
-                      (call-process gtags-global-command nil t nil option "--encode-path=\" \t\"" tagname))))
-	(progn (message (buffer-substring (point-min)(1- (point-max))))
-               (gtags-pop-context))
-      (goto-char (point-min))
-      (setq lines (count-lines (point-min) (point-max)))
+    (let (status)
+      (gtags-push-tramp-environment)
+      ;
+      ; Path style is defined in gtags-path-style:
+      ;   root: relative from the root of the project (Default)
+      ;   relative: relative from the current directory
+      ;	absolute: absolute (relative from the system root directory)
+      ; In TRAMP mode, 'root' is automatically converted to 'relative'.
+      ;
       (cond
-       ((= 0 lines)
-         (cond
-          ((char-equal flag-char ?P)
-           (message "%s: path not found" tagname))
-          ((char-equal flag-char ?g)
-           (message "%s: pattern not found" tagname))
-          ((char-equal flag-char ?I)
-           (message "%s: token not found" tagname))
-          ((char-equal flag-char ?s)
-           (message "%s: symbol not found" tagname))
-          (t
-           (message "%s: tag not found" tagname)))
-	(gtags-pop-context)
-	(kill-buffer buffer)
-	(set-buffer save))
-       ((= 1 lines)
-	(message "Searching %s ... Done" tagname)
-	(gtags-select-it t other-win))
-       (t
-        (if (null other-win)
-            (switch-to-buffer buffer)
-          (switch-to-buffer-other-window buffer))
-	(gtags-select-mode))))))
+       ((equal gtags-path-style 'absolute)
+        (setq option (concat option "a")))
+       ((and (not gtags-tramp-active) (equal gtags-path-style 'root))
+        (let (rootdir)
+          (if gtags-rootdir
+            (setq rootdir gtags-rootdir)
+           (setq rootdir (gtags-get-rootpath)))
+          (if rootdir (cd rootdir)))))
+      (setq status (if (equal flag "C")
+                      (call-process gtags-global-command nil t nil option "--encode-path=\" \t\"" context tagname)
+                      (call-process gtags-global-command nil t nil option "--encode-path=\" \t\"" tagname)))
+      (gtags-pop-tramp-environment)
+      (if (not (= 0 status))
+          (progn (message (buffer-substring (point-min)(1- (point-max))))
+            (gtags-pop-context))
+        (goto-char (point-min))
+        (setq lines (count-lines (point-min) (point-max)))
+        (cond
+         ((= 0 lines)
+           (cond
+            ((char-equal flag-char ?P)
+             (message "%s: path not found" tagname))
+            ((char-equal flag-char ?g)
+             (message "%s: pattern not found" tagname))
+            ((char-equal flag-char ?I)
+             (message "%s: token not found" tagname))
+            ((char-equal flag-char ?s)
+             (message "%s: symbol not found" tagname))
+            (t
+             (message "%s: tag not found" tagname)))
+	  (gtags-pop-context)
+          (kill-buffer buffer)
+          (set-buffer save))
+         ((= 1 lines)
+          (message "Searching %s ... Done" tagname)
+          (gtags-select-it t other-win))
+         (t
+          (if (null other-win)
+              (switch-to-buffer buffer)
+              (switch-to-buffer-other-window buffer))
+         (gtags-select-mode)))))))
 
 ;; select a tag line from lines
 (defun gtags-select-it (delete &optional other-win)
