@@ -62,6 +62,7 @@ static int compare_path(const void *, const void *);
 static int compare_lineno(const void *, const void *);
 static int compare_tags(const void *, const void *);
 static const char *seekto(const char *, int);
+static int is_defined_in_GTAGS(GTOP *, const char *);
 static void flush_pool(GTOP *, const char *);
 static void segment_read(GTOP *);
 
@@ -121,59 +122,14 @@ seekto(const char *string, int n)
 	}
 	return p;
 }
-/*
- * tmpfile_put: write to the temporary file instead of db(3) file.
- *
- *	i)	fp	file pointer
- *	i)	key	key of db(3)
- *	i)	data	data of db(3)
- *
- * record format:
- * <key>\t<data>\n
- *
- * Finally, this record should be written in db(3) file.
- */
-void
-tmpfile_put(FILE *fp, const char *key, const char *data)
-{
-	fputs(key, fp);
-	putc('\t', fp);
-	fputs(data, fp);
-	putc('\n', fp);
-}
 /*
  * Tag format
  *
- * [Specification of format version 4]
+ * [Specification of format version 6]
  * 
- * Standard format (for GTAGS)
- * 
- *         <file id> <tag name> <line number> <line image>
- * 
- *                 * Separator is single blank.
- * 
- *         [example]
- *         +------------------------------------
- *         |110 func 10 int func(int a)
- *         |110 func 30 func(int a1, int a2)
- * 
- *         Line image might be compressed (GTAGS_COMPRESS).
+ * Standard format:
  *
- * With compact option for GRTAGS, GSYMS (GTAGS_COMPACT)
- * 
- *         <file id> <tag name> <line number>,...
- * 
- *                 * Separator is single blank.
- * 
- *         [example]
- *         +------------------------------------
- *         |110 func 10,30
- * 
- *         Line numbers are sorted in a line.
- * 
- * [Specification of format version 5]
- * 
- * Standard format for GTAGS
+ *	This format is the default format of GTAGS.
  * 
  *         <file id> <tag name> <line number> <line image>
  * 
@@ -187,16 +143,20 @@ tmpfile_put(FILE *fp, const char *key, const char *data)
  *         Line image might be compressed (GTAGS_COMPRESS).
  *         Tag name might be compressed (GTAGS_COMPNAME).
  *
- * With compact option for GRTAGS, GSYMS (GTAGS_COMPACT)
+ * Compact format:
  * 
+ *	This format is the default format of GRTAGS.
+ *	It is used for GTAGS with the -c option.
+ *
  *         <file id> <tag name> <line number>,...
  * 
  *                 * Separator is single blank.
  * 
  *         [example]
  *         +------------------------------------
- *         |110 func 10,20
+ *         |110 func 10,30
  * 
+ *         Line numbers are sorted in a line.
  *	   Each line number might be expressed as difference from the previous
  *	   line number except for the head (GTAGS_COMPLINE).
  *           ex: 10,3,2 means '10 13 15'.
@@ -206,7 +166,8 @@ tmpfile_put(FILE *fp, const char *key, const char *data)
  * [Description]
  * 
  * o Standard format is applied to GTAGS, and compact format is applied
- *   to GRTAGS and GSYMS by default.
+ *   to GRTAGS by default.
+ * o GSYMS is not used any longer. It is virtually included by GRTAGS.
  * o Above two formats are same to the first line number. So, we can use
  *   common function to sort them.
  * o Separator is single blank.
@@ -246,8 +207,10 @@ tmpfile_put(FILE *fp, const char *key, const char *data)
  *                      if (format > 3) then print error message.
  * GLOBAL-5.0 -	5.3	support format version only 4.
  *                      if (format !=  4) then print error message.
- * GLOBAL-5.4 -		support format version 4 and 5
+ * GLOBAL-5.4 - 5.8.2	support format version 4 and 5
  *                      if (format > 5 || format < 4) then print error message.
+ * GLOBAL-5.9 -		support only format version 6
+ *                      if (format > 6 || format < 6) then print error message.
  *
  * In GLOBAL-5.0, we threw away the compatibility with the past formats.
  * Though we could continue the support for older formats, it seemed
@@ -261,9 +224,50 @@ tmpfile_put(FILE *fp, const char *key, const char *data)
  *       $ global -x main
  *       GTAGS seems older format. Please remake tag files.
  */
-static int upper_bound_version = 5;	/* acceptable format version (upper bound) */
-static int lower_bound_version = 4;	/* acceptable format version (lower bound) */
+static int new_format_version = 6;	/* new format version */
+static int upper_bound_version = 6;	/* acceptable format version (upper bound) */
+static int lower_bound_version = 6;	/* acceptable format version (lower bound) */
 static const char *const tagslist[] = {"GPATH", "GTAGS", "GRTAGS", "GSYMS"};
+/*
+ * Virtual GRTAGS, GSYMS processing:
+ *
+ * We use a real GRTAGS as virtual GRTAGS and GSYMS.
+ * In fact, GSYMS tag file doesn't exist.
+ *
+ * Real tag file	virtual tag file
+ * --------------------------------------
+ * GTAGS =============> GTAGS
+ *
+ * GRTAGS ============> GRTAGS + GSYMS
+ *            +=======> GRTAGS	tags which is defined in GTAGS
+ *            +=======> GSYMS	tags which is not defined in GTAGS
+ */
+#define VIRTUAL_GRTAGS_GSYMS_PROCESSING(gtop) 						\
+	if (gtop->db == GRTAGS || gtop->db == GSYMS) {					\
+		int defined = is_defined_in_GTAGS(gtop, gtop->dbop->lastkey);		\
+		if ((gtop->db == GRTAGS && !defined) || (gtop->db == GSYMS && defined))	\
+			continue;							\
+	}
+/*
+ * is_defined_in_GTAGS: whether or not the name is defined in GTAGS.
+ *
+ *	i)	gtop
+ *	i)	name	tag name
+ *	r)		0: not defined, 1: defined
+ *
+ * It is assumed that the input stream is sorted by the tag name.
+ */
+static int
+is_defined_in_GTAGS(GTOP *gtop, const char *name)
+{
+	static char prev_name[MAXTOKEN+1];
+	static int prev_result;
+
+	if (!strcmp(name, prev_name))
+		return prev_result;
+	strlimcpy(prev_name, name, sizeof(prev_name));
+	return prev_result = dbop_get(gtop->gtags, prev_name) ? 1 : 0;
+}
 /*
  * dbname: return db name
  *
@@ -273,6 +277,8 @@ static const char *const tagslist[] = {"GPATH", "GTAGS", "GRTAGS", "GSYMS"};
 const char *
 dbname(int db)
 {
+	if (db == GRTAGS + GSYMS)
+		db = GRTAGS;
 	assert(db >= 0 && db < GTAGLIM);
 	return tagslist[db];
 }
@@ -294,13 +300,13 @@ GTOP *
 gtags_open(const char *dbpath, const char *root, int db, int mode, int flags)
 {
 	GTOP *gtop;
+	char tagfile[MAXPATHLEN];
 	int dbmode;
 
 	gtop = (GTOP *)check_calloc(sizeof(GTOP), 1);
 	gtop->db = db;
 	gtop->mode = mode;
 	gtop->openflags = flags;
-	gtop->format_version = 4;
 	/*
 	 * Open tag file allowing duplicate records.
 	 */
@@ -317,18 +323,39 @@ gtags_open(const char *dbpath, const char *root, int db, int mode, int flags)
 	default:
 		assert(0);
 	}
-	gtop->dbop = dbop_open(makepath(dbpath, dbname(db), NULL), dbmode, 0644, DBOP_DUP);
+	/*
+	 * GRTAGS and GSYMS are virtual tag file. They are included in a real GRTAGS file.
+	 * In fact, GSYMS doesn't exist now.
+	 *
+	 * GRTAGS:	tags which belongs to GRTAGS, and are defined in GTAGS.
+	 * GSYMS:	tags which belongs to GRTAGS, and is not defined in GTAGS.
+	 */
+	strlimcpy(tagfile, makepath(dbpath, dbname(db == GSYMS ? GRTAGS : db), NULL), sizeof(tagfile));
+	gtop->dbop = dbop_open(tagfile, dbmode, 0644, DBOP_DUP|DBOP_SORTED_WRITE);
 	if (gtop->dbop == NULL) {
 		if (dbmode == 1)
 			die("cannot make %s.", dbname(db));
 		die("%s not found.", dbname(db));
 	}
+	if (gtop->mode == GTAGS_READ && db != GTAGS) {
+		const char *gtags = makepath(dbpath, dbname(GTAGS), NULL);
+		int format_version;
+
+		gtop->gtags = dbop_open(gtags, 0, 0, 0);
+		if (gtop->gtags == NULL)
+			die("GTAGS not found.");
+		format_version = dbop_getversion(gtop->dbop);
+		if (format_version > upper_bound_version)
+			die("%s seems new format. Please install the latest GLOBAL.", gtags);
+		else if (format_version < lower_bound_version)
+			die("%s seems older format. Please remake tag files.", gtags);
+	}
 	if (gtop->mode == GTAGS_CREATE) {
 		/*
 		 * Decide format.
 		 */
 		gtop->format = 0;
-		gtop->format_version = 5;
+		gtop->format_version = new_format_version;
 		/*
 		 * GRTAGS and GSYSM always use compact format.
 		 * GTAGS uses compact format only when the -c option specified.
@@ -365,9 +392,9 @@ gtags_open(const char *dbpath, const char *root, int db, int mode, int flags)
 		 */
 		gtop->format_version = dbop_getversion(gtop->dbop);
 		if (gtop->format_version > upper_bound_version)
-			die("%s seems new format. Please install the latest GLOBAL.", dbname(gtop->db));
+			die("%s seems new format. Please install the latest GLOBAL.", tagfile);
 		else if (gtop->format_version < lower_bound_version)
-			die("%s seems older format. Please remake tag files.", dbname(gtop->db));
+			die("%s seems older format. Please remake tag files.", tagfile);
 		gtop->format = 0;
 		if (dbop_getoption(gtop->dbop, COMPACTKEY) != NULL)
 			gtop->format |= GTAGS_COMPACT;
@@ -459,11 +486,7 @@ gtags_put_using(GTOP *gtop, const char *tag, int lno, const char *fid, const cha
 	strbuf_putn(gtop->sb, lno);
 	strbuf_putc(gtop->sb, ' ');
 	strbuf_puts(gtop->sb, (gtop->format & GTAGS_COMPRESS) ? compress(img, key) : img);
-	if (gtop->tmpfile_fp != NULL) {
-		tmpfile_put(gtop->tmpfile_fp, key, strbuf_value(gtop->sb));
-	} else {
-		dbop_put(gtop->dbop, key, strbuf_value(gtop->sb));
-	}
+	dbop_put(gtop->dbop, key, strbuf_value(gtop->sb));
 }
 /*
  * gtags_flush: Flush the pool for compact format.
@@ -479,178 +502,6 @@ gtags_flush(GTOP *gtop, const char *fid)
 		strhash_reset(gtop->path_hash);
 	}
 }
-/*
- * gtags_add_ref_sym: read candidate of reference and other symbol from temporary file,
- *                    and put into GRTAGS or GSYMS.
- *
- *	i) gtop		array of descripter of GTOP
- *	i) ip		file pointer of temporary file
- */
-void
-gtags_add_ref_sym(GTOP *const *gtop, FILE *ip)
-{
-	STRBUF *ib = strbuf_open(MAXBUFLEN);
-
-	rewind(ip);
-	while (strbuf_fgets(ib, ip, STRBUF_NOCRLF) != NULL) {
-		/*
-		 * [record format]
-		 * <key>\t<data>\n
-		 */
-		char *key = strbuf_value(ib);
-		char *data = strchr(key, '\t');
-		if (data == NULL)
-			die("gtags_add_ref_sym: internal error.");
-		*data++ = '\0';
-		/*
-		 * If the key is defined in GTAGS then put the record into GRTAGS
-		 * else put it into GSYMS.
-		 */
-		if (dbop_get(gtop[GTAGS]->dbop, key) != NULL)
-			dbop_put(gtop[GRTAGS]->dbop, key, data);
-		else
-			dbop_put(gtop[GSYMS]->dbop, key, data);
-	}
-	strbuf_close(ib);
-}
-/*
- * gtags_move_ref_sym: move defined symbols from GSYMS to GRTAGS,
- *                     and move undefined symbols from GRTAGS to GSYMS.
- *
- *	i) gtop		array of descripter of GTOP
- */
-void
-gtags_move_ref_sym(GTOP *const *gtop)
-{
-	int result = 0;
-	DBOP *defdbop;
-	const char *defkey;
-	struct {
-		DBOP *dbop;
-		const char *key, *dat;
-		char saved_key[MAXKEYLEN+1];
-		STRBUF *saved_dat;
-	} ref, sym, *smaller, *larger;
-
-	defdbop = gtop[GTAGS]->dbop;
-	defkey = dbop_first(defdbop, NULL, NULL, DBOP_KEY);
-	ref.dbop = gtop[GRTAGS]->dbop;
-	ref.dat = dbop_first(ref.dbop, NULL, NULL, 0);
-	ref.key = ref.dbop->lastkey;
-	ref.saved_dat = strbuf_open(MAXBUFLEN);
-	sym.dbop = gtop[GSYMS]->dbop;
-	sym.dat = dbop_first(sym.dbop, NULL, NULL, 0);
-	sym.key = sym.dbop->lastkey;
-	sym.saved_dat = strbuf_open(MAXBUFLEN);
-	for (;;) {
-		if (ref.dat != NULL && (sym.dat == NULL || strcmp(ref.key, sym.key) < 0)) {
-			smaller = &ref;
-			larger = &sym;
-		} else if (sym.dat != NULL) {
-			smaller = &sym;
-			larger = &ref;
-		} else {
-			break;
-		}
-		while (defkey != NULL && (result = strcmp(smaller->key, defkey)) > 0)
-			defkey = dbop_next(defdbop);
-		if (defkey != NULL && (smaller == &ref ? result < 0 : result == 0)) {
-			/*
-			 * Calling dbop_put doesn't affect the position of cursor,
-			 * but it invalidates the pointer to read data.
-			 */
-			if (larger->dat != NULL && larger->key != larger->saved_key) {
-				strlimcpy(larger->saved_key, larger->key, sizeof(larger->saved_key));
-				larger->key = larger->saved_key;
-				strbuf_reset(larger->saved_dat);
-				strbuf_puts(larger->saved_dat, larger->dat);
-				larger->dat = strbuf_value(larger->saved_dat);
-			}
-			dbop_put(larger->dbop, smaller->key, smaller->dat);
-			dbop_delete(smaller->dbop, NULL);
-		}
-		smaller->dat = dbop_next(smaller->dbop);
-		smaller->key = smaller->dbop->lastkey;
-	}
-
-	strbuf_close(ref.saved_dat);
-	strbuf_close(sym.saved_dat);
-}
-/*
- * gtags_put: put tag record with packing.
- *
- *	i)	gtop	descripter of GTOP
- *	i)	key	tag name
- *	i)	ctags_x	tag line (ctags -x format)
- */
-void
-gtags_put(GTOP *gtop, const char *key, const char *ctags_x)	/* virtually const */
-{
-	SPLIT ptable;
-
-	if (split((char *)ctags_x, 4, &ptable) != 4) {
-		recover(&ptable);
-		die("illegal tag format.\n'%s'", ctags_x);
-	}
-	if (gtop->format & GTAGS_COMPACT) {
-		int *lno;
-		struct sh_entry *entry;
-		/*
-		 * Flush the pool when path is changed.
-		 * Line numbers in the pool will be sorted and duplicated
-		 * records will be combined.
-		 *
-		 * pool    "funcA"   | 1| 3| 7|23|11| 2|...
-		 *           v
-		 * output  funcA 33 1,2,3,7,11,23...
-		 */
-		if (gtop->cur_path[0] && strcmp(gtop->cur_path, ptable.part[PART_PATH].start)) {
-			flush_pool(gtop, NULL);
-			strhash_reset(gtop->path_hash);
-		}
-		strlimcpy(gtop->cur_path, ptable.part[PART_PATH].start, sizeof(gtop->cur_path));
-		/*
-		 * Register each record into the pool.
-		 *
-		 * Pool image:
-		 *
-		 * tagname   lno
-		 * ------------------------------
-		 * "funcA"   | 1| 3| 7|23|11| 2|...
-		 * "funcB"   |34| 2| 5|66| 3|...
-		 * ...
-		 */
-		entry = strhash_assign(gtop->path_hash, key, 1);
-		if (entry->value == NULL)
-			entry->value = varray_open(sizeof(int), 100);
-		lno = varray_append((VARRAY *)entry->value);
-		*lno = atoi(ptable.part[PART_LNO].start);
-	} else {
-		const char *s_fid = gpath_path2fid(ptable.part[PART_PATH].start, NULL);
-
-		if (s_fid == NULL)
-			die("GPATH is corrupted.('%s' not found)", ptable.part[PART_PATH].start);
-		strbuf_reset(gtop->sb);
-		strbuf_puts(gtop->sb, s_fid);
-		strbuf_putc(gtop->sb, ' ');
-		if (gtop->format & GTAGS_COMPNAME)
-			strbuf_puts(gtop->sb, compress(ptable.part[PART_TAG].start, key));
-		else
-			strbuf_puts(gtop->sb, ptable.part[PART_TAG].start);
-		strbuf_putc(gtop->sb, ' ');
-		strbuf_puts(gtop->sb, ptable.part[PART_LNO].start);
-		strbuf_putc(gtop->sb, ' ');
-		strbuf_puts(gtop->sb, gtop->format & GTAGS_COMPRESS ?
-			compress(ptable.part[PART_LINE].start, key) :
-			ptable.part[PART_LINE].start);
-		if (gtop->tmpfile_fp != NULL) {
-			tmpfile_put(gtop->tmpfile_fp, key, strbuf_value(gtop->sb));
-		} else {
-			dbop_put(gtop->dbop, key, strbuf_value(gtop->sb));
-		}
-	}
-	recover(&ptable);
-}
 /*
  * gtags_delete: delete records belong to set of fid.
  *
@@ -696,7 +547,7 @@ gtags_first(GTOP *gtop, const char *pattern, int flags)
 {
 	int dbflags = 0;
 	int regflags = 0;
-	char prefix[IDENTLEN+1];
+	char prefix[IDENTLEN];
 	static regex_t reg;
 	regex_t *preg = &reg;
 	const char *key = NULL;
@@ -781,6 +632,7 @@ gtags_first(GTOP *gtop, const char *pattern, int flags)
 		     tagline != NULL;
 		     tagline = dbop_next(gtop->dbop))
 		{
+			VIRTUAL_GRTAGS_GSYMS_PROCESSING(gtop);
 			/* extract file id */
 			p = locatestring(tagline, " ", MATCH_FIRST);
 			if (p == NULL)
@@ -820,8 +672,14 @@ gtags_first(GTOP *gtop, const char *pattern, int flags)
 		gtop->gtp.path = gtop->path_array[gtop->path_index++];
 		return &gtop->gtp;
 	} else if (gtop->flags & GTOP_KEY) {
-		return ((gtop->gtp.tag = dbop_first(gtop->dbop, key, preg, dbflags)) == NULL)
-			? NULL : &gtop->gtp;
+		for (gtop->gtp.tag = dbop_first(gtop->dbop, key, preg, dbflags);
+		     gtop->gtp.tag != NULL;
+		     gtop->gtp.tag = dbop_next(gtop->dbop))
+		{
+			VIRTUAL_GRTAGS_GSYMS_PROCESSING(gtop);
+			break;
+		}
+		return gtop->gtp.tag ? &gtop->gtp : NULL;
 	} else {
 		if (gtop->vb == NULL)
 			gtop->vb = varray_open(sizeof(GTP), 200);
@@ -865,8 +723,14 @@ gtags_next(GTOP *gtop)
 		gtop->gtp.path = gtop->path_array[gtop->path_index++];
 		return &gtop->gtp;
 	} else if (gtop->flags & GTOP_KEY) {
-		return ((gtop->gtp.tag = dbop_next(gtop->dbop)) == NULL)
-			? NULL : &gtop->gtp;
+		for (gtop->gtp.tag = dbop_next(gtop->dbop);
+		     gtop->gtp.tag != NULL;
+		     gtop->gtp.tag = dbop_next(gtop->dbop))
+		{
+			VIRTUAL_GRTAGS_GSYMS_PROCESSING(gtop);
+			break;
+		}
+		return gtop->gtp.tag ? &gtop->gtp : NULL;
 	} else {
 		/*
 		 * End of segment.
@@ -907,6 +771,8 @@ gtags_close(GTOP *gtop)
 		strhash_close(gtop->path_hash);
 	gpath_close();
 	dbop_close(gtop->dbop);
+	if (gtop->gtags)
+		dbop_close(gtop->gtags);
 	free(gtop);
 }
 /*
@@ -1001,11 +867,7 @@ flush_pool(GTOP *gtop, const char *s_fid)
 						strbuf_putn(gtop->sb, n);
 					}
 					if (strbuf_getlen(gtop->sb) > DBOP_PAGESIZE / 4) {
-						if (gtop->tmpfile_fp != NULL) {
-							tmpfile_put(gtop->tmpfile_fp, key, strbuf_value(gtop->sb));
-						} else {
-							dbop_put(gtop->dbop, key, strbuf_value(gtop->sb));
-						}
+						dbop_put(gtop->dbop, key, strbuf_value(gtop->sb));
 						strbuf_setlen(gtop->sb, header_offset);
 					}
 				}
@@ -1029,22 +891,14 @@ flush_pool(GTOP *gtop, const char *s_fid)
 					strbuf_putc(gtop->sb, ',');
 				strbuf_putn(gtop->sb, n);
 				if (strbuf_getlen(gtop->sb) > DBOP_PAGESIZE / 4) {
-					if (gtop->tmpfile_fp != NULL) {
-						tmpfile_put(gtop->tmpfile_fp, key, strbuf_value(gtop->sb));
-					} else {
-						dbop_put(gtop->dbop, key, strbuf_value(gtop->sb));
-					}
+					dbop_put(gtop->dbop, key, strbuf_value(gtop->sb));
 					strbuf_setlen(gtop->sb, header_offset);
 				}
 				last = n;
 			}
 		}
 		if (strbuf_getlen(gtop->sb) > header_offset) {
-			if (gtop->tmpfile_fp != NULL) {
-				tmpfile_put(gtop->tmpfile_fp, key, strbuf_value(gtop->sb));
-			} else {
-				dbop_put(gtop->dbop, key, strbuf_value(gtop->sb));
-			}
+			dbop_put(gtop->dbop, key, strbuf_value(gtop->sb));
 		}
 		/* Free line number table */
 		varray_close(vb);
@@ -1081,6 +935,7 @@ segment_read(GTOP *gtop)
 	 */
 	gtop->cur_tagname[0] = '\0';
 	while ((tagline = dbop_next(gtop->dbop)) != NULL) {
+		VIRTUAL_GRTAGS_GSYMS_PROCESSING(gtop);
 		/*
 		 * get tag name and line number.
 		 *
