@@ -26,7 +26,7 @@ case $1 in
 	exit 0;;
 esac
 prog='autoreconf flex gperf perl bison'	# required programs
-file='convert.pl configure.ac Makefile.am gtags-parser/reserved.pl'	# required files
+file='convert.pl configure.ac Makefile.am libparser/reserved.pl'	# required files
 
 echo "- File existent checking..."
 for f in `echo $file`; do
@@ -62,7 +62,7 @@ done
 # flex, bison and gperf.
 #
 echo "- Preparing parser source ..."
-(cd gtags-parser; set -x
+(cd libparser; set -x
 for lang in c cpp java php asm; do
 	name=${lang}_res
 	perl ./reserved.pl --prefix=$lang ${lang}_res.in > ${name}.gpf
@@ -84,22 +84,9 @@ for lang in c cpp java php asm; do
 	flex -o$lang.c $lang.l
 done
 )
-(cd libparser; set -x
-for lang in c cpp java php asm; do
-	if [ -f $lang.l ]; then
-		flex -o$lang.c $lang.l
-	fi
-	if [ -f ${lang}_scan.l ]; then
-		flex -o${lang}_scan.c ${lang}_scan.l
-	fi
-	if [ -f ${lang}_parse.y ]; then
-		bison -d -o ${lang}_parse.c ${lang}_parse.y
-	fi
-done
-)
 
 echo "- Collecting reference manuals ..."
-commands="global gtags htags gtags-parser gozilla gtags-cscope";
+commands="global gtags htags gozilla gtags-cscope globash";
 perl ./convert.pl --menu $commands > doc/reference.texi
 for d in `echo $commands`; do
 	perl ./convert.pl --info $d/manual.in > doc/$d.ref
